package org.openarchitectureware.compiler.runtime.util;

import java.util.Comparator;
import java.util.List;

public class ParamTypeListComparator implements Comparator<List<Class<?>>> {
    /**
     * returns -1 if the second list of types is not assignable to the first
     * list of types returns 0 if the second list of types exactly matches the
     * first list of types returns 1 if the second list of types is assignable
     * to the first list of types
     */
    public int compare(final List<Class<?>> types1, final List<Class<?>> types2) {
        if (types1.size() != types2.size())
            return -1;

        boolean directMatch = true;
        for (int i = 0, x = types1.size(); i < x; i++) {
            final Class<?> type1 = types1.get(i);
            final Class<?> type2 = types2.get(i);
            if (type1.isAssignableFrom(type2) || Void.class.equals (type2)) {
                if (!type1.equals(type2)) {
                    directMatch = false;
                }
            } else
                return -1;
        }
        if (directMatch)
            return 0;
        else
            return 1;
    }

}
