/*******************************************************************************
 * Copyright (c) 2005 - 2007 committers of openArchitectureWare and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     committers of openArchitectureWare - initial API and implementation
 *******************************************************************************/
package org.openarchitectureware.expression.debug;

import static org.openarchitectureware.debug.processing.IEventHandler.NORMAL_FRAME;

import org.openarchitectureware.debug.model.SyntaxElementTO;
import org.openarchitectureware.expression.ExecutionContext;
import org.openarchitectureware.expression.ast.SyntaxElement;

/**
 * Adapter to handle statements that don't have an associated resource. (mainly the initial one)
 * 
 * @author Clemens Kadura (zAJKa)
 */
public class NoResourceSpecial extends BaseSpecialTreatment {

	/**
	 * Don't suspend for normal frame, if the element has no resource (virtual ones)
	 * 
	 * @see org.openarchitectureware.expression.debug.BaseSpecialTreatment#shallNotSuspend(java.lang.Object, int)
	 */
	@Override
	public boolean shallNotSuspend(Object element, int flag, ExecutionContext ctx) {
		return flag == NORMAL_FRAME && !hasResource(element);
	}

	/**
	 * Don't show frame in Launch view, if the element has no resource (virtual ones)
	 * 
	 * @see org.openarchitectureware.expression.debug.BaseSpecialTreatment#adaptSyntaxElementTO(org.openarchitectureware.debug.model.SyntaxElementTO,
	 *      java.lang.Object)
	 */
	@Override
	public void adaptSyntaxElementTO(SyntaxElementTO to, Object element) {
		if (!hasResource(element))
			to.visible = false;
	}

	// -------------------------------------------------------------------------

	private boolean hasResource(Object element) {
		return getClass().getResource("/" + ((SyntaxElement) element).getFileName()) != null;
	}

}
