/*
 * <copyright>
 *
 * Copyright (c) 2005-2006 Sven Efftinge (http://www.efftinge.de) and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sven Efftinge (http://www.efftinge.de) - Initial API and implementation
 *
 * </copyright>
 */
package org.openarchitectureware.xtend.ast;

import java.util.Set;

import org.openarchitectureware.expression.AnalysationIssue;
import org.openarchitectureware.expression.ExecutionContext;
import org.openarchitectureware.expression.ast.Identifier;
import org.openarchitectureware.expression.ast.SyntaxElement;
import org.openarchitectureware.xtend.XtendFile;

public class ImportStatement extends SyntaxElement {

	private Identifier importedId;

	private boolean exported;

	public ImportStatement(final Identifier importedID, final boolean exported) {
		importedId = importedID;
		this.exported = exported;
	}

	public Identifier getImportedId() {
		return importedId;
	}

	public boolean isExported() {
		return exported;
	}

	public void analyze(ExecutionContext ctx, Set<AnalysationIssue> issues) {
		try {
			if (ctx.getCallback() != null)
				ctx.getCallback().pre(this, ctx);
			final XtendFile xf = (XtendFile) ctx.getResourceManager().loadResource(this.getImportedId().getValue(),
					XtendFile.FILE_EXTENSION);
			if (xf == null) {
				final String msg = "Error while importing extension: File " + this.getImportedId().getValue() + " not found.";
				issues.add(new AnalysationIssue(AnalysationIssue.RESOURCE_NOT_FOUND, msg, this));
				return;
			}
		} finally {
			if (ctx.getCallback() != null)
				ctx.getCallback().post(null);
		}
	}

}
