/*
 * <copyright>
 *
 * Copyright (c) 2005-2006 Sven Efftinge (http://www.efftinge.de) and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sven Efftinge (http://www.efftinge.de) - Initial API and implementation
 *
 * </copyright>
 */
package org.openarchitectureware.xtend.core.internal;

import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IStorage;
import org.openarchitectureware.core.AbstractOawResource;
import org.openarchitectureware.expression.AnalysationIssue;
import org.openarchitectureware.expression.ExecutionContext;
import org.openarchitectureware.expression.Resource;
import org.openarchitectureware.expression.ResourceManager;
import org.openarchitectureware.xtend.XtendFile;
import org.openarchitectureware.xtend.ast.Around;
import org.openarchitectureware.xtend.ast.Extension;
import org.openarchitectureware.xtend.core.IXtendResource;
import org.openarchitectureware.xtend.core.internal.builder.XtendResourceParser;

public class XtendResourceImpl extends AbstractOawResource implements IXtendResource {

    private XtendResourceParser parser;

	public XtendResourceImpl(final XtendFile tpl, final IStorage res, XtendResourceParser parser) {
        super(res);
        setOawResource(tpl);
        this.parser = parser;
    }

    private XtendFile resource() {
        return (XtendFile) getOawResource();
    }

    public void analyze(final ExecutionContext ctx, final Set<AnalysationIssue> issues) {
    	try {
    		resource().analyze(ctx, issues);
    	} catch (Exception e) {
    		// ignore
    	}
    }

    public List<Extension> getExtensions() {
        return resource().getExtensions();
    }

    public List<Extension> getPublicExtensions(final ResourceManager rm, ExecutionContext ctx) {
        return resource().getPublicExtensions(rm, ctx);
    }

    public List<Extension> getPublicExtensions(ResourceManager resourceManager, ExecutionContext ctx, Set<String> flowoverCache) {
        return resource().getPublicExtensions(resourceManager,ctx, flowoverCache);
    }

	public String getFileExtension() {
		return parser.getFileExtension();
	}

	public boolean internalRefresh() {
		Resource r = parser.parse(getUnderlyingStorage(),getFullyQualifiedName());
		if (r==null) {
			return false;
		}
		setOawResource(r);
		return true;
	}

	@SuppressWarnings("unchecked")
	public List<Around> getArounds() {
		return Collections.EMPTY_LIST; // arounds are only used at runtime
	}
}
