package org.openarchitectureware.xtext.parser;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;
import org.eclipse.emf.ecore.EObject;
import org.openarchitectureware.check.CheckFacade;
import org.openarchitectureware.emf.EcoreUtil2;
import org.openarchitectureware.expression.ExecutionContext;
import org.openarchitectureware.workflow.issues.Issues;
import org.openarchitectureware.workflow.issues.IssuesImpl;
import org.openarchitectureware.xtend.XtendFacade;
import org.openarchitectureware.xtext.parser.antlr.XTextLexer;
import org.openarchitectureware.xtext.parser.antlr.XTextParser;
import org.openarchitectureware.xtext.parser.impl.AbstractXtextParser;
import org.openarchitectureware.xtext.parser.parsetree.Node;

public class GenParser extends AbstractXtextParser implements IXtextParser {

	private InputStreamReader reader;
	private XTextLexer lexer;
	private XTextParser parser;
	private Node result;

	public GenParser(InputStream in) {
		this.reader = new InputStreamReader(in);
	}

	public XTextLexer getLexer() {
		if (lexer == null) {
			try {
				this.lexer = new XTextLexer(new ANTLRReaderStream(reader));
			} catch (IOException e) {
				handleError(e);
			}
		}
		return lexer;
	}

	protected void handleError(Exception e) {
		throw new RuntimeException(e);
	}

	public XTextParser getParser() {
		if (parser == null) {
			parser = new XTextParser(new CommonTokenStream(getLexer()));
		}
		return parser;
	}

	public List<ErrorMsg> getParseErrors() {
		List<ErrorMsg> errors = new ArrayList<ErrorMsg>();
		errors.addAll(getLexer().getErrors());
		errors.addAll(getParser().getErrors());
		return errors;
	}

	public Node getRootNode() {
		if (result == null) {
			try {
				result = getParser().parse();
			} catch (RecognitionException e) {
				handleError(e);
			}
		}
		return result;
	}

	@Override
	protected void internalPostLink() {

	}

	@Override
	protected void internalPreLink() {
		ExecutionContext ctx = getExecutionContext();
		XtendFacade facade = XtendFacade.create(ctx,
				"org::openarchitectureware::xtext::XtextExtensions");
		facade.call("fixTypes", getRootNode().getModelElement());
	}

	@Override
	protected void internalLink() {
		if (!getParseErrors().isEmpty())
			return;
		ExecutionContext ctx = getExecutionContext();
		XtendFacade facade = XtendFacade.create(ctx,
				"org::openarchitectureware::xtext::parser::Linker");
		facade.call("link", getRootNode().getModelElement());
	}

	public Issues doCheck() {
		ExecutionContext ctx = getExecutionContext();
		EObject rootElement = getRootNode().getModelElement();
		Collection<EObject> allElements = EcoreUtil2.allContents(rootElement);
		allElements.add(rootElement);
		IssuesImpl issues = new IssuesImpl();
		String[] checkFiles = new String[] {
				"org::openarchitectureware::xtext::GenChecks",
				"org::openarchitectureware::xtext::Checks" };
		for (String checkFile : checkFiles) {
			CheckFacade.checkAll(checkFile, allElements, ctx, issues);
		}
		return issues;
	}

}
